@@ -1,9 +1,34 @@
+/*******************************************************************************
+ qtprivate.h
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/
+
 #ifndef PRIVATE_H
 #define PRIVATE_H
 
+#include "charset.h"
+#include <quicktime/lqt_atoms.h>
 #include <inttypes.h>
-#include <lqt_atoms.h>
-#include <stdio.h>
+#include <stdio.h> // For quicktime_s->stream
 
 /* ================================= structures */
 
@@ -348,6 +373,56 @@ typedef struct
   float gamma;
   } quicktime_gama_t;
 
+/* Font table for MPEG-4 timed text */
+
+typedef struct
+  {
+  uint16_t num_fonts;
+
+  struct
+    {
+    uint16_t font_id;
+    char font_name[256];
+    } * fonts;
+  } quicktime_ftab_t;
+
+/* Sample description for Quicktime text tracks */
+
+typedef struct
+  {
+  uint32_t displayFlags;
+  uint32_t textJustification;
+  uint16_t bgColor[3];
+  uint16_t defaultTextBox[4];
+  uint32_t scrpStartChar;              /*starting character position*/
+  uint16_t scrpHeight;
+  uint16_t scrpAscent;
+  uint16_t scrpFont;
+  uint16_t scrpFace;
+  uint16_t scrpSize;
+  uint16_t scrpColor[3];
+  char font_name[256];
+  } quicktime_stsd_text_t;
+
+/* Sample description for MPEG-4 text tracks */
+
+typedef struct
+  {
+  uint32_t display_flags;
+  uint8_t horizontal_justification;
+  uint8_t vertical_justification;
+  uint8_t back_color[4];
+  uint16_t defaultTextBox[4];
+  uint16_t start_char_offset;
+  uint16_t end_char_offset;
+  uint16_t font_id;
+  uint8_t  style_flags;
+  uint8_t  font_size;
+  uint8_t  text_color[4];
+  int has_ftab;
+  quicktime_ftab_t ftab;
+  } quicktime_stsd_tx3g_t;
+
 typedef struct
 {
 	char format[4];
@@ -426,7 +501,11 @@ typedef struct
         unsigned char * table_raw;
         int table_raw_size;
 
-        
+/* Quicktime text */
+        quicktime_stsd_text_t text;
+
+/* Quicktime tx3g */
+        quicktime_stsd_tx3g_t tx3g;
 
 } quicktime_stsd_table_t;
 
@@ -560,10 +639,17 @@ typedef struct
   } quicktime_stbl_t;
 
 typedef struct
-{
-    	char refType[4];
-	long trackIndex;
-} quicktime_tref_t;
+  {
+  char type[4];
+  int num_tracks;
+  uint32_t * tracks;
+  } quicktime_track_reference_t;
+
+typedef struct
+  {
+  int num_references;
+  quicktime_track_reference_t * references;
+  } quicktime_tref_t;
 
 /* data reference */
 
@@ -625,14 +711,32 @@ typedef struct
 	int reserved;
 } quicktime_gmin_t;
 
+/* Obscure text atom found inside the gmhd atom
+ * of text tracks
+ * TODO: Reverse engineer this
+ */
 
-/* Base media header */
+typedef struct
+  {
+  uint32_t unk[9];
+  } quicktime_gmhd_text_t;
+
+
+/* Base (generic) media header */
+
+typedef struct
+  {
+  quicktime_gmin_t gmin;
+
+  int has_gmhd_text;
+  quicktime_gmhd_text_t gmhd_text;
+  } quicktime_gmhd_t;
 
 typedef struct
 {
-    	quicktime_gmin_t gmin;
-} quicktime_gmhd_t;
-
+	int version;
+	long flags;
+} quicktime_nmhd_t;
 
 /* handler reference */
 
@@ -655,16 +759,23 @@ typedef struct
 	int is_video;
 	int is_audio;
         int is_audio_vbr;   /* Special flag indicating VBR audio */
-	int has_baseheader;
 	int is_panorama;
 	int is_qtvr;
 	int is_object;
+        int is_text;
 	quicktime_vmhd_t vmhd;
 	quicktime_smhd_t smhd;
 	quicktime_gmhd_t gmhd;
-	quicktime_stbl_t stbl;
+	int has_gmhd;
+
+        quicktime_nmhd_t nmhd;
+	int has_nmhd;
+
+        quicktime_stbl_t stbl;
 	quicktime_hdlr_t hdlr;
-	quicktime_dinf_t dinf;
+        int has_hdlr;
+
+        quicktime_dinf_t dinf;
 } quicktime_minf_t;
 
 
@@ -743,18 +854,20 @@ typedef struct {
 
 
 typedef struct
-{
-	quicktime_tkhd_t tkhd;
-	quicktime_mdia_t mdia;
-	quicktime_edts_t edts;
-	quicktime_tref_t tref;
-        quicktime_strl_t * strl; // != NULL for AVI files during reading and writing
-        int chunk_sizes_alloc;
-        int64_t * chunk_sizes; /* This contains the chunk sizes for audio
-                                  tracks. They can not so easily be obtained
-                                  during decoding */
-	int has_tref;
-} quicktime_trak_t;
+  {
+  quicktime_tkhd_t tkhd;
+  quicktime_mdia_t mdia;
+  quicktime_edts_t edts;
+  int has_edts;
+  
+  quicktime_tref_t tref;
+  quicktime_strl_t * strl; // != NULL for AVI files during reading and writing
+  int chunk_sizes_alloc;
+  int64_t * chunk_sizes; /* This contains the chunk sizes for audio
+                            tracks. They can not so easily be obtained
+                            during decoding */
+  int has_tref;
+  } quicktime_trak_t;
 
 
 typedef struct
@@ -1178,6 +1291,8 @@ typedef struct
 /* WAV ID: Taken from the codec_info and saved here for writing the AVI header */
         int wav_id;
 
+        int compatibility_flags; /* Taken from codec info */
+
 } quicktime_audio_map_t;
 
 typedef struct
@@ -1205,11 +1320,32 @@ typedef struct
 
         /* This is used by the core to do implicit colorspace conversion and scaling (NOT recommended!!) */
         uint8_t ** temp_frame;
+
         lqt_chroma_placement_t chroma_placement;
         lqt_interlace_mode_t interlace_mode;
 
+        int compatibility_flags; /* Taken from codec info */
+
   } quicktime_video_map_t;
 
+/* Text track */
+
+typedef struct
+  {
+  quicktime_trak_t *track;
+  int is_chapter_track; /* For encoding only */
+  int current_position;
+  
+  lqt_charset_converter_t * cnv;
+
+  char * text_buffer;
+  int text_buffer_alloc;
+
+  int initialized; /* For encoding only */
+  int64_t current_chunk;
+  
+  } quicktime_text_map_t;
+
 /* obji */
 
 typedef struct
@@ -1373,12 +1509,26 @@ struct quicktime_s
 	int total_vtracks;
 	quicktime_video_map_t *vtracks;
 
+/* Mapping of text tracks to movie tracks */
+	int total_ttracks;
+	quicktime_text_map_t *ttracks;
+
 /* Parameters for frame currently being decoded */
 	int in_x, in_y, in_w, in_h, out_w, out_h;
 /*      Libquicktime change: color_model and row_span are now saved per track */
 /*	int color_model, row_span; */
 
 	quicktime_qtvr_node_t qtvr_node[MAXNODES];
+
+/* Logging support */
+        lqt_log_callback_t log_callback;
+        void * log_data;
+
+/* I/O Error is saved here: It has the advantage, that
+   codecs don't have to check the return values of quicktime_[read|write]_data
+   all the time. */
+        int io_error;
+        int io_eof;
 };
 
 typedef struct
