@@ -1,7 +1,206 @@
-#include <funcprotos.h>
-#include <quicktime/quicktime.h>
+/*******************************************************************************
+ stsdtable.c
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
+#include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 
+#define LOG_DOMAIN "stsdtable"
+
+static void quicktime_stsdtable_read_text(quicktime_t *file,
+                                          quicktime_stsd_table_t *table,
+                                          quicktime_atom_t *parent_atom)
+  {
+  table->text.displayFlags = quicktime_read_int32(file);
+  table->text.textJustification = quicktime_read_int32(file);
+
+  table->text.bgColor[0] = quicktime_read_int16(file);
+  table->text.bgColor[1] = quicktime_read_int16(file);
+  table->text.bgColor[2] = quicktime_read_int16(file);
+
+  table->text.defaultTextBox[0] = quicktime_read_int16(file);
+  table->text.defaultTextBox[1] = quicktime_read_int16(file);
+  table->text.defaultTextBox[2] = quicktime_read_int16(file);
+  table->text.defaultTextBox[3] = quicktime_read_int16(file);
+
+  table->text.scrpStartChar = quicktime_read_int32(file);
+
+  table->text.scrpHeight = quicktime_read_int16(file);
+  table->text.scrpAscent = quicktime_read_int16(file);
+  table->text.scrpFont = quicktime_read_int16(file);
+  table->text.scrpFace = quicktime_read_int16(file);
+  table->text.scrpSize = quicktime_read_int16(file);
+  
+  table->text.scrpColor[0] = quicktime_read_int16(file);
+  table->text.scrpColor[1] = quicktime_read_int16(file);
+  table->text.scrpColor[2] = quicktime_read_int16(file);
+
+  quicktime_read_pascal(file, table->text.font_name);
+  }
+
+static void quicktime_write_stsd_text(quicktime_t *file,
+                                      quicktime_stsd_table_t *table)
+  {
+  quicktime_write_int32(file, table->text.displayFlags);
+  quicktime_write_int32(file, table->text.textJustification);
+
+  quicktime_write_int16(file, table->text.bgColor[0]);
+  quicktime_write_int16(file, table->text.bgColor[1]);
+  quicktime_write_int16(file, table->text.bgColor[2]);
+
+  quicktime_write_int16(file, table->text.defaultTextBox[0]);
+  quicktime_write_int16(file, table->text.defaultTextBox[1]);
+  quicktime_write_int16(file, table->text.defaultTextBox[2]);
+  quicktime_write_int16(file, table->text.defaultTextBox[3]);
+
+  quicktime_write_int32(file, table->text.scrpStartChar);
+
+  quicktime_write_int16(file, table->text.scrpHeight);
+  quicktime_write_int16(file, table->text.scrpAscent);
+  quicktime_write_int16(file, table->text.scrpFont);
+  quicktime_write_int16(file, table->text.scrpFace);
+  quicktime_write_int16(file, table->text.scrpSize);
+  
+  quicktime_write_int16(file, table->text.scrpColor[0]);
+  quicktime_write_int16(file, table->text.scrpColor[1]);
+  quicktime_write_int16(file, table->text.scrpColor[2]);
+
+  quicktime_write_pascal(file, table->text.font_name);
+  }
+
+static void quicktime_stsdtable_dump_text(quicktime_stsd_table_t *table)
+  {
+  lqt_dump("       displayFlags      %08x\n", table->text.displayFlags);
+  lqt_dump("       textJustification %d\n", table->text.textJustification);
+  lqt_dump("       bgColor:          [%d,%d,%d]\n", table->text.bgColor[0],
+           table->text.bgColor[1],table->text.bgColor[2]);
+  lqt_dump("       defaultTextBox:   [%d,%d,%d,%d]\n",
+           table->text.defaultTextBox[0],
+           table->text.defaultTextBox[1],
+           table->text.defaultTextBox[2],
+           table->text.defaultTextBox[3]);
+  lqt_dump("       scrpStartChar:    %d\n", table->text.scrpStartChar);
+  lqt_dump("       scrpHeight:       %d\n", table->text.scrpHeight);
+  lqt_dump("       scrpFont:         %d\n", table->text.scrpFont);
+  lqt_dump("       scrpFace:         %d\n", table->text.scrpFace);
+  lqt_dump("       scrpSize:         %d\n", table->text.scrpSize);
+  lqt_dump("       scrpColor:        [%d,%d,%d]\n",
+           table->text.scrpColor[0],
+           table->text.scrpColor[1],
+           table->text.scrpColor[2]);
+  lqt_dump("       Font:             %s\n", table->text.font_name);
+  }
+
+static void
+quicktime_stsdtable_read_tx3g(quicktime_t *file,
+                              quicktime_stsd_table_t *table,
+                              quicktime_atom_t *parent_atom)
+  {
+  quicktime_atom_t leaf_atom;
+  table->tx3g.display_flags = quicktime_read_int32(file);
+  quicktime_read_data(file, &table->tx3g.horizontal_justification, 1);
+  quicktime_read_data(file, &table->tx3g.vertical_justification, 1);
+  quicktime_read_data(file, table->tx3g.back_color, 4);
+  table->tx3g.defaultTextBox[0] = quicktime_read_int16(file);
+  table->tx3g.defaultTextBox[1] = quicktime_read_int16(file);
+  table->tx3g.defaultTextBox[2] = quicktime_read_int16(file);
+  table->tx3g.defaultTextBox[3] = quicktime_read_int16(file);
+  table->tx3g.start_char_offset = quicktime_read_int16(file);
+  table->tx3g.end_char_offset   = quicktime_read_int16(file);
+  table->tx3g.font_id           = quicktime_read_int16(file);
+  quicktime_read_data(file, &table->tx3g.style_flags, 1);
+  quicktime_read_data(file, &table->tx3g.font_size, 1);
+  quicktime_read_data(file, table->tx3g.text_color, 4);
+
+  while(quicktime_position(file) < parent_atom->end)
+    {
+    quicktime_atom_read_header(file, &leaf_atom);
+    if(quicktime_atom_is(&leaf_atom, "ftab"))
+      {
+      quicktime_read_ftab(file, &(table->tx3g.ftab));
+      table->tx3g.has_ftab = 1;
+      }
+    else
+      quicktime_atom_skip(file, &leaf_atom);
+    }
+  }
+
+static void
+quicktime_write_stsd_tx3g(quicktime_t *file,
+                          quicktime_stsd_table_t *table)
+  {
+  quicktime_write_int32(file, table->tx3g.display_flags);
+  quicktime_write_data(file, &table->tx3g.horizontal_justification, 1);
+  quicktime_write_data(file, &table->tx3g.vertical_justification, 1);
+  quicktime_write_data(file, table->tx3g.back_color, 4);
+  quicktime_write_int16(file, table->tx3g.defaultTextBox[0]);
+  quicktime_write_int16(file, table->tx3g.defaultTextBox[1]);
+  quicktime_write_int16(file, table->tx3g.defaultTextBox[2]);
+  quicktime_write_int16(file, table->tx3g.defaultTextBox[3]);
+  quicktime_write_int16(file, table->tx3g.start_char_offset);
+  quicktime_write_int16(file, table->tx3g.end_char_offset);
+  quicktime_write_int16(file, table->tx3g.font_id);
+  quicktime_write_data(file, &table->tx3g.style_flags, 1);
+  quicktime_write_data(file, &table->tx3g.font_size, 1);
+  quicktime_write_data(file, table->tx3g.text_color, 4);
+  
+  if(table->tx3g.has_ftab)
+    quicktime_write_ftab(file, &table->tx3g.ftab);
+  }
+
+static void quicktime_stsdtable_dump_tx3g(quicktime_stsd_table_t *table)
+  {
+  lqt_dump("       display_flags:            %08x\n", table->tx3g.display_flags);
+  lqt_dump("       horizontal_justification: %d\n", table->tx3g.horizontal_justification);
+  lqt_dump("       vertical_justification:   %d\n", table->tx3g.vertical_justification);
+  lqt_dump("       back_color:               [%d,%d,%d,%d]\n",
+           table->tx3g.back_color[0],
+           table->tx3g.back_color[1],
+           table->tx3g.back_color[2],
+           table->tx3g.back_color[3]);
+  lqt_dump("       defaultTextBox:           [%d,%d,%d,%d]\n",
+           table->tx3g.defaultTextBox[0],
+           table->tx3g.defaultTextBox[1],
+           table->tx3g.defaultTextBox[2],
+           table->tx3g.defaultTextBox[3]);
+  lqt_dump("       start_char_offset:        %d\n", table->tx3g.start_char_offset);
+  lqt_dump("       end_char_offset:          %d\n", table->tx3g.end_char_offset);
+  lqt_dump("       font_id:                  %d\n", table->tx3g.font_id);
+  lqt_dump("       style_flags:              %02x\n", table->tx3g.style_flags);
+  lqt_dump("       font_size:                %d\n", table->tx3g.font_size);
+  lqt_dump("       text_color:               [%d,%d,%d,%d]\n",
+           table->tx3g.text_color[0], table->tx3g.text_color[1],
+           table->tx3g.text_color[2], table->tx3g.text_color[3]);
+
+  if(table->tx3g.has_ftab)
+    {
+    quicktime_ftab_dump(&table->tx3g.ftab);
+    }
+  }
+
+
 void quicktime_read_stsd_audio(quicktime_t *file, quicktime_stsd_table_t *table,
                                quicktime_atom_t *parent_atom)
 {
@@ -21,7 +220,6 @@ void quicktime_read_stsd_audio(quicktime_t *file, quicktime_stsd_table_t *table,
           if(table->samplerate + 65536 == 96000 ||
              table->samplerate + 65536 == 88200) table->samplerate += 65536;
           
-          //          fprintf(stderr, "stsd version: %d\n", table->version);
           
           if(table->version == 1)
             {
@@ -47,7 +245,6 @@ void quicktime_read_stsd_audio(quicktime_t *file, quicktime_stsd_table_t *table,
            */
           quicktime_set_position(file, quicktime_position(file) + 16);
           //          quicktime_set_position(file, quicktime_position(file) + 12);
-          //          fprintf(stderr, "sizeOfStructOnly: %d\n", quicktime_read_int32(file));
           
           /*
            * Float64    audioSampleRate;
@@ -65,14 +262,11 @@ void quicktime_read_stsd_audio(quicktime_t *file, quicktime_stsd_table_t *table,
            */
           
           quicktime_set_position(file, quicktime_position(file) + 4);
-          //          fprintf(stderr, "always7F000000: %08lx\n", quicktime_read_int32(file));
 
           
 
           table->sample_size = quicktime_read_int32(file);
-          //          fprintf(stderr, "constBitsPerChannel: %d\n", table->sample_size);
           table->formatSpecificFlags = quicktime_read_int32(file);
-          //          fprintf(stderr, "formatSpecificFlags: %08x\n", table->formatSpecificFlags);
 
           /* The following 2 are (hopefully) unused */
           
@@ -98,18 +292,16 @@ void quicktime_read_stsd_audio(quicktime_t *file, quicktime_stsd_table_t *table,
             quicktime_read_esds(file, &(table->esds));
             table->has_esds = 1;
             quicktime_atom_skip(file, &leaf_atom);
-            // fprintf(stderr, "Got esds\n");
             }
           else if(quicktime_atom_is(&leaf_atom, "chan"))
             {
             quicktime_read_chan(file, &(table->chan));
             table->has_chan = 1;
             quicktime_atom_skip(file, &leaf_atom);
-            //            fprintf(stderr, "Got chan\n");
             }
           else
             {
-            fprintf(stderr, "Skipping unknown atom \"%4s\" inside audio stsd\n",
+            lqt_log(file, LQT_LOG_INFO, LOG_DOMAIN, "Skipping unknown atom \"%4s\" inside audio stsd",
                     leaf_atom.type);
             quicktime_atom_skip(file, &leaf_atom);
             }
@@ -209,7 +401,6 @@ void quicktime_read_stsd_video(quicktime_t *file, quicktime_stsd_table_t *table,
 	quicktime_atom_t leaf_atom;
 	int len, bits_per_pixel;
         
-        //        fprintf(stderr, "quicktime_read_stsd_video 1 %lld\n", quicktime_position(file));
 	table->version = quicktime_read_int16(file);
 	table->revision = quicktime_read_int16(file);
 	quicktime_read_data(file, (uint8_t*)table->vendor, 4);
@@ -226,7 +417,6 @@ void quicktime_read_stsd_video(quicktime_t *file, quicktime_stsd_table_t *table,
 	table->depth = quicktime_read_int16(file);
 	table->ctab_id = quicktime_read_int16(file);
         
-        //  fprintf(stderr, "quicktime_read_stsd_video 2 %lld\n", quicktime_position(file));
         
         /*  If ctab_id is zero, the colortable follows immediately
          *  after the ctab ID
@@ -238,27 +428,14 @@ void quicktime_read_stsd_video(quicktime_t *file, quicktime_stsd_table_t *table,
             (bits_per_pixel == 4) ||
             (bits_per_pixel == 8)))
           {
-          //          fprintf(stderr, "Reading color table, depth: %d\n", table->depth);
           quicktime_read_ctab(file, &(table->ctab));
           }
         else
           quicktime_default_ctab(&(table->ctab), table->depth);
         
-        //        fprintf(stderr, "quicktime_read_stsd_video 3 %lld\n",
-        //                quicktime_position(file));
 	while(quicktime_position(file) + 8 < parent_atom->end)
 	{
 		quicktime_atom_read_header(file, &leaf_atom);
-#if 0
-                fprintf(stderr, "quicktime_read_stsd_video 1 %c%c%c%c, pos: %lld, end: %lld\n",
-                        leaf_atom.type[0],
-                        leaf_atom.type[1],
-                        leaf_atom.type[2],
-                        leaf_atom.type[3],
-                        quicktime_position(file),
-                        parent_atom->end
-                       );
-#endif		
 		if(quicktime_atom_is(&leaf_atom, "ctab"))
 		{
 			quicktime_read_ctab(file, &(table->ctab));
@@ -296,7 +473,6 @@ void quicktime_read_stsd_video(quicktime_t *file, quicktime_stsd_table_t *table,
 		else
 		if (quicktime_atom_is(&leaf_atom, "esds"))
 		{
-                //                fprintf(stderr, "*** GOT ESDS ATOM\n");
 			quicktime_read_esds(file, &(table->esds));
                         table->has_esds = 1;
                         quicktime_atom_skip(file, &leaf_atom);
@@ -309,18 +485,7 @@ void quicktime_read_stsd_video(quicktime_t *file, quicktime_stsd_table_t *table,
                                                &leaf_atom);
                 }
                 quicktime_atom_skip(file, &leaf_atom);
-#if 0
-                fprintf(stderr, "quicktime_read_stsd_video 2 %c%c%c%c, pos: %lld, end: %lld\n",
-                leaf_atom.type[0],
-                leaf_atom.type[1],
-                leaf_atom.type[2],
-                leaf_atom.type[3],
-                quicktime_position(file),
-                parent_atom->end
-                );
-#endif
 	}
-//printf("quicktime_read_stsd_video 2\n");
 }
 
 
@@ -414,6 +579,16 @@ void quicktime_read_stsd_table(quicktime_t *file, quicktime_minf_t *minf,
 	{	    
 	    minf->is_object = 1;
 	}
+        else if (quicktime_match_32(leaf_atom.type, "text"))
+        {
+            quicktime_stsdtable_read_text(file, table, &leaf_atom);
+            minf->is_text = 1;
+        }
+        else if (quicktime_match_32(leaf_atom.type, "tx3g"))
+        {
+            quicktime_stsdtable_read_tx3g(file, table, &leaf_atom);
+            minf->is_text = 1;
+        }
 	else 
 	{
 	    if(minf->is_audio) quicktime_read_stsd_audio(file, table, &leaf_atom);
@@ -475,25 +650,26 @@ void quicktime_stsd_table_delete(quicktime_stsd_table_t *table)
         quicktime_ctab_delete(&(table->ctab));
 	quicktime_wave_delete(&(table->wave));
 	quicktime_esds_delete(&(table->esds));
+	quicktime_ftab_delete(&(table->tx3g.ftab));
         quicktime_user_atoms_delete(&(table->user_atoms));
 }
 
 void quicktime_stsd_video_dump(quicktime_stsd_table_t *table)
 {
-	printf("       version %d\n", table->version);
-	printf("       revision %d\n", table->revision);
-	printf("       vendor %c%c%c%c\n", table->vendor[0], table->vendor[1], table->vendor[2], table->vendor[3]);
-	printf("       temporal_quality %ld\n", table->temporal_quality);
-	printf("       spatial_quality %ld\n", table->spatial_quality);
-	printf("       width %d\n", table->width);
-	printf("       height %d\n", table->height);
-	printf("       dpi_horizontal %f\n", table->dpi_horizontal);
-	printf("       dpi_vertical %f\n", table->dpi_vertical);
-	printf("       data_size %lld\n", table->data_size);
-	printf("       frames_per_sample %d\n", table->frames_per_sample);
-	printf("       compressor_name %s\n", table->compressor_name);
-	printf("       depth %d\n", table->depth);
-	printf("       ctab_id %d\n", table->ctab_id);
+	lqt_dump("       version %d\n", table->version);
+	lqt_dump("       revision %d\n", table->revision);
+	lqt_dump("       vendor %c%c%c%c\n", table->vendor[0], table->vendor[1], table->vendor[2], table->vendor[3]);
+	lqt_dump("       temporal_quality %ld\n", table->temporal_quality);
+	lqt_dump("       spatial_quality %ld\n", table->spatial_quality);
+	lqt_dump("       width %d\n", table->width);
+	lqt_dump("       height %d\n", table->height);
+	lqt_dump("       dpi_horizontal %f\n", table->dpi_horizontal);
+	lqt_dump("       dpi_vertical %f\n", table->dpi_vertical);
+	lqt_dump("       data_size %lld\n", table->data_size);
+	lqt_dump("       frames_per_sample %d\n", table->frames_per_sample);
+	lqt_dump("       compressor_name %s\n", table->compressor_name);
+	lqt_dump("       depth %d\n", table->depth);
+	lqt_dump("       ctab_id %d\n", table->ctab_id);
 
 	if (table->has_pasp)
 		quicktime_pasp_dump(&(table->pasp));
@@ -513,33 +689,33 @@ void quicktime_stsd_video_dump(quicktime_stsd_table_t *table)
 
 void quicktime_stsd_audio_dump(quicktime_stsd_table_t *table)
 {
-	printf("       version %d\n", table->version);
-	printf("       revision %d\n", table->revision);
-	printf("       vendor %c%c%c%c\n", table->vendor[0], table->vendor[1], table->vendor[2], table->vendor[3]);
-	printf("       channels %d\n", table->channels);
-	printf("       sample_size %d\n", table->sample_size);
+	lqt_dump("       version %d\n", table->version);
+	lqt_dump("       revision %d\n", table->revision);
+	lqt_dump("       vendor %c%c%c%c\n", table->vendor[0], table->vendor[1], table->vendor[2], table->vendor[3]);
+	lqt_dump("       channels %d\n", table->channels);
+	lqt_dump("       sample_size %d\n", table->sample_size);
 
         if(table->version < 2)
           {
-          printf("       compression_id %d\n", table->compression_id);
-          printf("       packet_size %d\n", table->packet_size);
-          printf("       samplerate %f\n",  table->samplerate);
+          lqt_dump("       compression_id %d\n", table->compression_id);
+          lqt_dump("       packet_size %d\n", table->packet_size);
+          lqt_dump("       samplerate %f\n",  table->samplerate);
           
           if(table->version == 1)
             {
-            printf("       samples_per_packet: %d\n", table->audio_samples_per_packet);
-            printf("       bytes_per_packet:   %d\n", table->audio_bytes_per_packet);
-            printf("       bytes_per_frame:    %d\n", table->audio_bytes_per_frame);
-            printf("       bytes_per_samples:  %d\n", table->audio_bytes_per_sample);
+            lqt_dump("       samples_per_packet: %d\n", table->audio_samples_per_packet);
+            lqt_dump("       bytes_per_packet:   %d\n", table->audio_bytes_per_packet);
+            lqt_dump("       bytes_per_frame:    %d\n", table->audio_bytes_per_frame);
+            lqt_dump("       bytes_per_samples:  %d\n", table->audio_bytes_per_sample);
             }
           
           }
         else if(table->version == 2)
           {
-          printf("       samplerate                     %f\n",  table->samplerate);
-          printf("       formatSpecificFlags:           %08x\n", table->formatSpecificFlags);
-          printf("       constBytesPerAudioPacket:      %d\n", table->constBytesPerAudioPacket);
-          printf("       constLPCMFramesPerAudioPacket: %d\n", table->constLPCMFramesPerAudioPacket);
+          lqt_dump("       samplerate                     %f\n",  table->samplerate);
+          lqt_dump("       formatSpecificFlags:           %08x\n", table->formatSpecificFlags);
+          lqt_dump("       constBytesPerAudioPacket:      %d\n", table->constBytesPerAudioPacket);
+          lqt_dump("       constLPCMFramesPerAudioPacket: %d\n", table->constLPCMFramesPerAudioPacket);
           }
         
         if(table->has_wave)
@@ -555,35 +731,46 @@ void quicktime_stsd_audio_dump(quicktime_stsd_table_t *table)
 void quicktime_stsd_table_dump(void *minf_ptr, quicktime_stsd_table_t *table)
 {
 	quicktime_minf_t *minf = minf_ptr;
-	printf("       format %c%c%c%c\n",
+	lqt_dump("       format %c%c%c%c\n",
                table->format[0], table->format[1],
                table->format[2], table->format[3]);
 
 	quicktime_print_chars("       reserved ", table->reserved, 6);
-	printf("       data_reference %d\n", table->data_reference);
+	lqt_dump("       data_reference %d\n", table->data_reference);
 
 	if(minf->is_audio) quicktime_stsd_audio_dump(table);
-	if(minf->is_video) quicktime_stsd_video_dump(table);
-
-	if (quicktime_match_32(table->format, "pano"))
+	else if(minf->is_video) quicktime_stsd_video_dump(table);
+        
+	else if (quicktime_match_32(table->format, "pano"))
 	    quicktime_pano_dump(&(table->pano));
-	if (quicktime_match_32(table->format, "qtvr"))
+	else if (quicktime_match_32(table->format, "qtvr"))
 	    quicktime_qtvr_dump(&(table->qtvr));
+        else if(quicktime_match_32(table->format, "text"))
+            quicktime_stsdtable_dump_text(table);
+        else if(quicktime_match_32(table->format, "tx3g"))
+            quicktime_stsdtable_dump_tx3g(table);
 }
 
 void quicktime_write_stsd_table(quicktime_t *file, quicktime_minf_t *minf, quicktime_stsd_table_t *table)
 {
 	quicktime_atom_t atom;
 	quicktime_atom_write_header(file, &atom, table->format);
-/*printf("quicktime_write_stsd_table %c%c%c%c\n", table->format[0], table->format[1], table->format[2], table->format[3]); */
 	quicktime_write_data(file, table->reserved, 6);
 	quicktime_write_int16(file, table->data_reference);
 
 	if(minf->is_audio) quicktime_write_stsd_audio(file, table);
 	if(minf->is_video) quicktime_write_stsd_video(file, table);
-	if(minf->is_panorama) quicktime_write_pano(file, &(table->pano));
-	if(minf->is_qtvr == QTVR_QTVR) quicktime_write_qtvr(file, &(table->qtvr));
+	if(minf->is_qtvr == QTVR_QTVR_PAN) quicktime_write_qtvr(file, &(table->qtvr));
+	if(minf->is_qtvr == QTVR_QTVR_OBJ) quicktime_write_qtvr(file, &(table->qtvr));
 
+        if(minf->is_text)
+          {
+          if(quicktime_match_32(table->format, "text"))
+            quicktime_write_stsd_text(file, table);
+          else if(quicktime_match_32(table->format, "tx3g"))
+            quicktime_write_stsd_tx3g(file, table);
+          }
+        
 	quicktime_atom_write_footer(file, &atom);
 }
 
