@@ -1,8 +1,29 @@
-#include <quicktime/quicktime.h>
-#include <funcprotos.h>
+/*******************************************************************************
+ trak.c
 
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
 
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
 
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
+#include <stdlib.h>
 
 int quicktime_trak_init(quicktime_trak_t *trak)
 {
@@ -34,8 +55,9 @@ int quicktime_trak_init_video(quicktime_t *file,
                 frame_duration,
                 timescale,
 		compressor);
-
-	return 0;
+	quicktime_edts_init_table(&(trak->edts));
+        trak->has_edts = 1;
+        return 0;
 }
 
 
@@ -43,14 +65,15 @@ int quicktime_trak_init_qtvr(quicktime_t *file, quicktime_trak_t *trak, int trac
 {
 
 	quicktime_tkhd_init_video(file, 
-		&(trak->tkhd), 
+		&(trak->tkhd),
 		height, 
-		height/2);
+		width);
 	quicktime_mdia_init_qtvr(file, 
-		&(trak->mdia), track_type, width, height, timescale, frame_duration);
+		&(trak->mdia), track_type, timescale, frame_duration);
 	quicktime_edts_init_table(&(trak->edts));
+        trak->has_edts = 1;
 	quicktime_tref_init_qtvr(&(trak->tref), track_type);
-
+        trak->has_tref = 1;
 	return 0;
 }
 
@@ -64,27 +87,53 @@ int quicktime_trak_init_panorama(quicktime_t *file, quicktime_trak_t *trak, int
 	quicktime_mdia_init_panorama(file, 
 		&(trak->mdia), width, height, timescale, frame_duration);
 	quicktime_edts_init_table(&(trak->edts));
+        trak->has_edts = 1;
 
 	return 0;
 }
 
-
-
 int quicktime_trak_init_audio(quicktime_t *file, 
 							quicktime_trak_t *trak, 
 							int channels, 
 							int sample_rate, 
 							int bits, 
 							char *compressor)
-{
-	quicktime_mdia_init_audio(file, &(trak->mdia), 
-		channels, 
-		sample_rate, 
-		bits, 
-		compressor);
-	quicktime_edts_init_table(&(trak->edts));
-	return 0;
-}
+  {
+  quicktime_mdia_init_audio(file, &(trak->mdia), 
+                            channels, 
+                            sample_rate, 
+                            bits, 
+                            compressor);
+  quicktime_edts_init_table(&(trak->edts));
+  trak->has_edts = 1;
+
+  return 0;
+  }
+
+int quicktime_trak_init_text(quicktime_t * file, quicktime_trak_t * trak,
+                             int timescale)
+  {
+  trak->tkhd.volume = 0;
+  trak->tkhd.flags = 3;
+  quicktime_mdia_init_text(file, &(trak->mdia), 
+                           timescale);
+  quicktime_edts_init_table(&(trak->edts));
+  trak->has_edts = 1;
+  return 0;
+  }
+
+int quicktime_trak_init_tx3g(quicktime_t * file, quicktime_trak_t * trak,
+                             int timescale)
+  {
+  trak->tkhd.volume = 0;
+  trak->tkhd.flags = 1;
+  quicktime_mdia_init_tx3g(file, &(trak->mdia), 
+                           timescale);
+ 
+  return 0;
+  }
+                            
+                            
 
 int quicktime_trak_delete(quicktime_trak_t *trak)
 {
@@ -101,9 +150,9 @@ int quicktime_trak_delete(quicktime_trak_t *trak)
 
 int quicktime_trak_dump(quicktime_trak_t *trak)
 {
-	printf(" track\n");
+	lqt_dump(" track\n");
 	quicktime_tkhd_dump(&(trak->tkhd));
-	quicktime_edts_dump(&(trak->edts));
+	if(trak->has_edts) quicktime_edts_dump(&(trak->edts));
 	if (trak->has_tref)
 	    quicktime_tref_dump(&(trak->tref));
 	quicktime_mdia_dump(&(trak->mdia));
@@ -136,50 +185,46 @@ int quicktime_delete_trak(quicktime_moov_t *moov)
 }
 
 
-int quicktime_read_trak(quicktime_t *file, quicktime_trak_t *trak, quicktime_atom_t *trak_atom)
-{
-	quicktime_atom_t leaf_atom;
+int quicktime_read_trak(quicktime_t *file, quicktime_trak_t *trak,
+                        quicktime_atom_t *trak_atom)
+  {
+  quicktime_atom_t leaf_atom;
 
-	do
-	{
-		quicktime_atom_read_header(file, &leaf_atom);
+  do
+    {
+    quicktime_atom_read_header(file, &leaf_atom);
 
-//printf("quicktime_read_trak %llx %llx\n", quicktime_position(file), quicktime_ftell(file));
-/* mandatory */
-		if(quicktime_atom_is(&leaf_atom, "tkhd"))
-			{ quicktime_read_tkhd(file, &(trak->tkhd)); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "mdia"))
-                  { quicktime_read_mdia(file, trak, &(trak->mdia), &leaf_atom); }
-		else
-/* optional */
-		if(quicktime_atom_is(&leaf_atom, "clip"))
-			{ quicktime_atom_skip(file, &leaf_atom); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "matt"))
-			{ quicktime_atom_skip(file, &leaf_atom); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "edts"))
-			{ quicktime_read_edts(file, &(trak->edts), &leaf_atom); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "load"))
-			{ quicktime_atom_skip(file, &leaf_atom); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "imap"))
-			{ quicktime_atom_skip(file, &leaf_atom); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "udta"))
-			{ quicktime_atom_skip(file, &leaf_atom); }
-		else
-		if(quicktime_atom_is(&leaf_atom, "tref"))
-			{ trak->has_tref = 1; quicktime_read_tref(file, &(trak->tref)); }
-		else
-			quicktime_atom_skip(file, &leaf_atom);
-//printf("quicktime_read_trak %llx %llx\n", quicktime_position(file), leaf_atom.end);
-	}while(quicktime_position(file) < trak_atom->end);
-
-	return 0;
-}
+    /* mandatory */
+    if(quicktime_atom_is(&leaf_atom, "tkhd"))
+      quicktime_read_tkhd(file, &(trak->tkhd));
+    else if(quicktime_atom_is(&leaf_atom, "mdia"))
+      quicktime_read_mdia(file, trak, &(trak->mdia), &leaf_atom);
+    /* optional */
+    else if(quicktime_atom_is(&leaf_atom, "clip"))
+      quicktime_atom_skip(file, &leaf_atom);
+    else if(quicktime_atom_is(&leaf_atom, "matt"))
+      quicktime_atom_skip(file, &leaf_atom);
+    else if(quicktime_atom_is(&leaf_atom, "edts"))
+      {
+      quicktime_read_edts(file, &(trak->edts), &leaf_atom);
+      trak->has_edts = 1;
+      }
+    else if(quicktime_atom_is(&leaf_atom, "load"))
+      quicktime_atom_skip(file, &leaf_atom);
+    else if(quicktime_atom_is(&leaf_atom, "imap"))
+      quicktime_atom_skip(file, &leaf_atom);
+    else if(quicktime_atom_is(&leaf_atom, "udta"))
+      quicktime_atom_skip(file, &leaf_atom);
+    else if(quicktime_atom_is(&leaf_atom, "tref"))
+      {
+      trak->has_tref = 1;
+      quicktime_read_tref(file, &(trak->tref), &leaf_atom);
+      }
+    else quicktime_atom_skip(file, &leaf_atom);
+    } while(quicktime_position(file) < trak_atom->end);
+  
+  return 0;
+  }
 
 int quicktime_write_trak(quicktime_t *file, 
 	quicktime_trak_t *trak, 
@@ -190,8 +235,7 @@ int quicktime_write_trak(quicktime_t *file,
 	quicktime_atom_t atom;
 	quicktime_atom_write_header(file, &atom, "trak");
 	quicktime_trak_duration(trak, &duration, &timescale);
-
-/*printf("quicktime_write_trak duration %d\n", duration); */
+        
 /* get duration in movie's units */
 	trak->tkhd.duration = (long)((float)duration / timescale * moov_time_scale);
 	trak->mdia.mdhd.duration = duration;
@@ -199,12 +243,12 @@ int quicktime_write_trak(quicktime_t *file,
 
 	quicktime_write_tkhd(file, &(trak->tkhd));
 	if (trak->mdia.minf.is_panorama) trak->edts.elst.total_entries = 1;
-	quicktime_write_edts(file, &(trak->edts), trak->tkhd.duration);
+        
+        if(trak->has_edts) quicktime_write_edts(file, &(trak->edts), trak->tkhd.duration);
 	quicktime_write_mdia(file, &(trak->mdia));
 	
-	if (trak->mdia.minf.is_qtvr) 
-	    quicktime_write_tref(file, &(trak->tref));
-//	printf("isobj: %i",trak->mdia.minf.is_object );
+	if (trak->has_tref) 
+          quicktime_write_tref(file, &(trak->tref));
 //	if (trak->mdia.minf.is_object) 
 //	    quicktime_write_tref(file, &(trak->tref));
 	
@@ -219,7 +263,6 @@ int64_t quicktime_track_samples(quicktime_t *file, quicktime_trak_t *trak)
 		int i;
 		int64_t total = 0;
 
-/*printf("file->rd %d file->wr %d\n", file->rd, file->wr); */
 	if(file->wr)
 	{
         if(trak->mdia.minf.is_audio)
@@ -463,8 +506,6 @@ void quicktime_write_chunk_header(quicktime_t *file,
     if(file->file_type == LQT_FILE_AVI_ODML)
       {
       quicktime_riff_t *riff = file->riff[file->total_riffs - 1];
-      //      fprintf(stderr, "Write chunk header %lld %d\n",
-      //              quicktime_position(file) - riff->atom.start, file->max_riff_size);
       if(quicktime_position(file) - riff->atom.start > file->max_riff_size)
         {
         quicktime_finalize_riff(file, riff);
@@ -482,10 +523,10 @@ void quicktime_write_chunk_header(quicktime_t *file,
   }
 
 void quicktime_write_chunk_footer(quicktime_t *file, 
-	quicktime_trak_t *trak,
-	int current_chunk,
-	quicktime_atom_t *chunk, 
-	int samples)
+                                  quicktime_trak_t *trak,
+                                  int current_chunk,
+                                  quicktime_atom_t *chunk, 
+                                  int samples)
 {
 	int64_t offset = chunk->start;
 	int sample_size = quicktime_position(file) - offset;
@@ -508,8 +549,6 @@ void quicktime_write_chunk_footer(quicktime_t *file,
           if(file->file_type == LQT_FILE_AVI_ODML)
             quicktime_update_ixtable(file, trak, offset, sample_size);
           
-          //          fprintf(stderr, "quicktime_write_chunk_footer: %d %d\n",
-          //                  sample_size, trak->strl->strh.dwSuggestedBufferSize);
           
           if(sample_size > trak->strl->strh.dwSuggestedBufferSize)
             trak->strl->strh.dwSuggestedBufferSize = ((sample_size+15)/16)*16;
@@ -521,10 +560,10 @@ void quicktime_write_chunk_footer(quicktime_t *file,
 		current_chunk, 
 		offset);
 
-	if(trak->mdia.minf.is_video)
-		quicktime_update_stsz(&(trak->mdia.minf.stbl.stsz), 
-		current_chunk - 1, 
-		sample_size);
+	if(trak->mdia.minf.is_video || trak->mdia.minf.is_text)
+          quicktime_update_stsz(&(trak->mdia.minf.stbl.stsz), 
+                                current_chunk - 1, 
+                                sample_size);
         /* Need to increase sample count for VBR (the VBR routines to it
            themselves) */
 	if(trak->mdia.minf.is_audio && !trak->mdia.minf.is_audio_vbr)
@@ -568,7 +607,7 @@ int quicktime_trak_duration(quicktime_trak_t *trak,
 int quicktime_trak_fix_counts(quicktime_t *file, quicktime_trak_t *trak)
 {
 	long samples = quicktime_track_samples(file, trak);
-        if(trak->mdia.minf.is_video)
+        if(trak->mdia.minf.is_video || trak->mdia.minf.is_text)
           {
           quicktime_compress_stts(&(trak->mdia.minf.stbl.stts));
           if(trak->mdia.minf.stbl.stts.total_entries == 1)
@@ -600,7 +639,6 @@ long quicktime_chunk_samples(quicktime_trak_t *trak, long chunk)
 
         if(!stsc->total_entries)
           return 0;
-        //        fprintf(stderr, "quicktime_chunk_samples: total_entries: %ld\n", stsc->total_entries);
 	do
 	{
 		current_chunk = stsc->table[i].chunk;
